@@ -2,16 +2,19 @@
 //!
 //! The Python bindings are implemented entirely in Rust using PyO3.
 
-use pyo3::exceptions::{PyNotImplementedError, PyTypeError};
+// FIXME: Add a workaround for rustc-1.57 and pyo3-0.15.1 combination.
+#![allow(clippy::needless_option_as_deref)]
+
+use pyo3::exceptions::{PyNotImplementedError, PyTypeError, PyValueError};
 use pyo3::types::PyBytes;
 
 use pyo3::prelude::*;
 
-use planetarium::{Matrix, Matrix23, Pixel, Point, Vector};
+use planetarium::{EncoderError, Matrix, Matrix23, Pixel, Point, Vector};
 
 use planetarium::{
     Canvas as RsCanvas, ImageFormat as RsImageFormat, SpotId as RsSpotId, SpotShape as RsSpotShape,
-    Transform as RsTransform,
+    Transform as RsTransform, Window as RsWindow,
 };
 
 /// Spot shape definition matrix
@@ -88,6 +91,33 @@ struct SpotId(RsSpotId);
 #[pyclass(module = "pyplanetarium", freelist = 8)]
 struct Transform(RsTransform);
 
+/// Canvas image window coordinates
+///
+/// Defines a rectangular window on the canvas to export the image from.
+///
+/// The window origin is defined by its upper left corner.
+///
+/// The window object can be constructed from a nested tuple `((x, y), (w, h))`,
+/// where `(w, h)` is the window rectangle dimensions and `(x, y)` is
+/// the window origin coordinates.
+///
+/// Example usage:
+///
+/// ```python
+/// from pyplanetarium import Window
+///
+/// # Create a new 128x64 window with origin at (100, 200).
+/// wnd1 = Window(((100, 200), (128, 64)));
+///
+/// # Create a new rectangular window with origin at (0, 0).
+/// wnd2 = Window.new(128, 64);
+///
+/// # Move the window origin to (250, 150).
+/// wnd3 = wnd2.at(250, 150);
+/// ```
+#[pyclass(module = "pyplanetarium", freelist = 8)]
+struct Window(RsWindow);
+
 /// Exportable canvas image formats
 ///
 /// Python class wrapper for Rust `ImageFormat` enum:
@@ -261,6 +291,37 @@ impl Transform {
     }
 }
 
+#[pymethods]
+impl Window {
+    #[new]
+    fn new_(src: ((u32, u32), (u32, u32))) -> Self {
+        Window(src.into())
+    }
+
+    /// Creates a new window with given dimensions located at the origin.
+    #[staticmethod]
+    #[pyo3(text_signature = "(width, height, /)")]
+    fn new(width: u32, height: u32) -> Self {
+        Window(RsWindow::new(width, height))
+    }
+
+    /// Moves the window origin to the given origin coordinates.
+    #[pyo3(text_signature = "(x, y, /)")]
+    fn at(&self, x: u32, y: u32) -> Window {
+        Window(self.0.at(x, y))
+    }
+
+    /// Implements `str(x)` in Python.
+    fn __str__(&self) -> String {
+        self.0.to_string()
+    }
+
+    /// Implements `repr(x)` in Python.
+    fn __repr__(&self) -> String {
+        format!("{:?}", self.0)
+    }
+}
+
 #[allow(non_upper_case_globals)]
 #[pymethods]
 impl ImageFormat {
@@ -295,8 +356,20 @@ impl ImageFormat {
     }
 }
 
+/// Converts `EncoderError` to Python exceptions.
+fn my_to_pyerr(err: EncoderError) -> PyErr {
+    match err {
+        EncoderError::BrokenWindow => PyValueError::new_err("window is out of bounds".to_string()),
+        _ => PyNotImplementedError::new_err(err.to_string()),
+    }
+}
+
 #[pymethods]
 impl Canvas {
+    /// `Pixel::MAX` alias
+    #[classattr]
+    const PIXEL_MAX: Pixel = Pixel::MAX;
+
     /// Creates a new clear canvas to render light spots on.
     #[staticmethod]
     #[pyo3(text_signature = "(width, height, /)")]
@@ -368,6 +441,9 @@ impl Canvas {
     }
 
     /// Sets the background light level (dark pixel value).
+    ///
+    /// The dark pixel value must be in the range
+    /// 0 to `Canvas.PIXEL_MAX` inclusive.
     #[pyo3(text_signature = "($self, level, /)")]
     fn set_background(&mut self, level: Pixel) {
         self.0.set_background(level);
@@ -392,7 +468,21 @@ impl Canvas {
     fn export_image(&self, format: &ImageFormat, py: Python) -> PyResult<Py<PyBytes>> {
         match self.0.export_image(format.0) {
             Ok(b) => Ok(PyBytes::new(py, b.as_slice()).into()),
-            Err(e) => Err(PyNotImplementedError::new_err(e.to_string())),
+            Err(e) => Err(my_to_pyerr(e)),
+        }
+    }
+
+    /// Exports the canvas window contents in the requested image format.
+    #[pyo3(text_signature = "($self, window, format, /)")]
+    fn export_window_image(
+        &self,
+        window: &Window,
+        format: &ImageFormat,
+        py: Python,
+    ) -> PyResult<Py<PyBytes>> {
+        match self.0.export_window_image(window.0, format.0) {
+            Ok(b) => Ok(PyBytes::new(py, b.as_slice()).into()),
+            Err(e) => Err(my_to_pyerr(e)),
         }
     }
 
@@ -416,7 +506,7 @@ impl Canvas {
 /// -------------
 ///
 /// ```python
-/// from pyplanetarium import Canvas, SpotShape, ImageFormat
+/// from pyplanetarium import Canvas, SpotShape, ImageFormat, Window
 ///
 /// # Draw on a square 256x256 pixel canvas.
 /// c = Canvas.new(256, 256)
@@ -435,7 +525,7 @@ impl Canvas {
 /// spot3 = c.add_spot((256.1, 3.5), shape.scale(10.0), 1.1)
 ///
 /// # Set the canvas background pixel value.
-/// c.set_background(100)
+/// c.set_background(int(0.1 * Canvas.PIXEL_MAX))
 ///
 /// # Clear the canvas and paint the light spots.
 /// c.draw()
@@ -445,6 +535,10 @@ impl Canvas {
 ///
 /// # Export to a 16-bit linear light grayscale PNG image.
 /// png_16bpp_bytes = c.export_image(ImageFormat.PngLinear16Bpp)
+///
+/// # Export a rectangular canvas window to a 8-bit gamma-compressed RAW image.
+/// wnd = Window.new(64, 32).at(90, 120)
+/// raw_window_8bpp_bytes = c.export_window_image(wnd, ImageFormat.PngGamma8Bpp)
 /// ```
 #[pymodule]
 fn pyplanetarium(_py: Python, m: &PyModule) -> PyResult<()> {
@@ -455,6 +549,7 @@ fn pyplanetarium(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_class::<SpotShape>()?;
     m.add_class::<SpotId>()?;
     m.add_class::<Transform>()?;
+    m.add_class::<Window>()?;
     m.add_class::<ImageFormat>()?;
     m.add_class::<Canvas>()?;
 
