@@ -4,7 +4,7 @@ pyplanetarium package integration tests
 
 import unittest
 
-from pyplanetarium import SpotShape, SpotId, Transform, Canvas, ImageFormat
+from pyplanetarium import SpotShape, SpotId, Transform, Canvas, ImageFormat, Window
 
 
 class CanvasCase(unittest.TestCase):
@@ -14,49 +14,9 @@ class CanvasCase(unittest.TestCase):
 
     def test_init(self) -> None:
         """
-        SpotShape and Canvas instantiation test
+        Canvas object instantiation test
         """
 
-        shape1 = SpotShape()
-        self.assertIsInstance(shape1, SpotShape)
-        self.assertEqual(str(shape1), "[[1, 0], [0, 1]]")
-        self.assertEqual(
-            repr(shape1), "SpotShape { xx: 1.0, xy: 0.0, yx: 0.0, yy: 1.0 }"
-        )
-
-        shape2 = shape1.scale(3.5)
-        self.assertNotEqual(shape1, shape2)
-
-        shape3 = SpotShape(3.5)
-        self.assertIsInstance(shape3, SpotShape)
-        self.assertEqual(str(shape3), "[[3.5, 0], [0, 3.5]]")
-        self.assertEqual(
-            repr(shape3), "SpotShape { xx: 3.5, xy: 0.0, yx: 0.0, yy: 3.5 }"
-        )
-
-        shape4 = SpotShape((3.5, 2.5))
-        self.assertIsInstance(shape4, SpotShape)
-        self.assertEqual(str(shape4), "[[3.5, 0], [0, 2.5]]")
-        self.assertEqual(
-            repr(shape4), "SpotShape { xx: 3.5, xy: 0.0, yx: 0.0, yy: 2.5 }"
-        )
-
-        shape5 = SpotShape([[3.5, 0.5], [-0.5, 2.5]])
-        self.assertIsInstance(shape5, SpotShape)
-        self.assertEqual(str(shape5), "[[3.5, 0.5], [-0.5, 2.5]]")
-        self.assertEqual(
-            repr(shape5), "SpotShape { xx: 3.5, xy: 0.5, yx: -0.5, yy: 2.5 }"
-        )
-
-        shape6 = SpotShape(3)
-        self.assertIsInstance(shape6, SpotShape)
-
-        shape7 = SpotShape((3, 2))
-        self.assertIsInstance(shape7, SpotShape)
-
-        shape8 = SpotShape([[3, 0], [0, 2]])
-        self.assertIsInstance(shape8, SpotShape)
-
         width = 1024
         height = 768
 
@@ -72,42 +32,9 @@ class CanvasCase(unittest.TestCase):
 
     def test_init_err(self) -> None:
         """
-        SpotShape and Canvas instantiation errors test
+        Canvas instantiation errors test
         """
 
-        with self.assertRaises(TypeError):
-            SpotShape({})  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape("1")  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape((1,))  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([1])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([1, 2])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([1, 2, 3])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([[1, 2], 2])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([[1, 2], [2]])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([[1, 2], [2, 3, 4]])  # type: ignore
-
-        with self.assertRaises(TypeError):
-            SpotShape([[1, 2], [2, 3], 4])  # type: ignore
-
         with self.assertRaises(TypeError):
             Canvas.new()  # type: ignore
 
@@ -167,7 +94,7 @@ class CanvasCase(unittest.TestCase):
 
         self.assertNotEqual(spot1, spot2)
 
-        canvas.set_background(1000)
+        canvas.set_background(int(0.1 * Canvas.PIXEL_MAX))
 
         canvas.draw()
 
@@ -203,7 +130,7 @@ class CanvasCase(unittest.TestCase):
         canvas.set_spot_offset(spot2, (5.5, -7.0))
         canvas.set_spot_illumination(spot2, 0.5)
 
-        canvas.set_background(6000)
+        canvas.set_background(int(0.2 * Canvas.PIXEL_MAX))
         canvas.set_brightness(1.3)
 
         pos2 = canvas.spot_position(spot2)
@@ -359,6 +286,51 @@ class CanvasCase(unittest.TestCase):
         # with open("image16.png", "wb") as f:
         #     f.write(png16_bytes)
 
+    def test_export_window_images(self) -> None:
+        """
+        Windowed canvas image export test
+        """
+
+        shape1 = SpotShape().scale(3.5)
+        shape2 = SpotShape().scale(5.5)
+
+        width = 256
+        height = 256
+
+        canvas = Canvas.new(width, height)
+
+        spot1 = canvas.add_spot((180.5, 150.7), shape1, 0.8)
+        spot2 = canvas.add_spot((100.5, 110.7), shape2, 0.6)
+
+        self.assertNotEqual(spot1, spot2)
+
+        canvas.set_background(5000)
+
+        canvas.draw()
+
+        wnd1 = Window.new(32, 16).at(170, 140)
+        wnd2 = Window.new(32, 16).at(90, 100)
+
+        raw8_bytes = canvas.export_window_image(wnd1, ImageFormat.RawGamma8Bpp)
+        self.assertIsInstance(raw8_bytes, bytes)
+        self.assertEqual(len(raw8_bytes), 32 * 16)
+
+        raw10_bytes = canvas.export_window_image(wnd1, ImageFormat.RawLinear10BppLE)
+        self.assertIsInstance(raw10_bytes, bytes)
+        self.assertEqual(len(raw10_bytes), 2 * 32 * 16)
+
+        raw12_bytes = canvas.export_window_image(wnd2, ImageFormat.RawLinear12BppLE)
+        self.assertIsInstance(raw12_bytes, bytes)
+        self.assertEqual(len(raw12_bytes), 2 * 32 * 16)
+
+        png8_bytes = canvas.export_window_image(wnd1, ImageFormat.PngGamma8Bpp)
+        self.assertIsInstance(png8_bytes, bytes)
+        self.assertEqual(len(png8_bytes), 250)
+
+        png16_bytes = canvas.export_window_image(wnd2, ImageFormat.PngLinear16Bpp)
+        self.assertIsInstance(png16_bytes, bytes)
+        self.assertEqual(len(png16_bytes), 664)
+
 
 if __name__ == "__main__":
     unittest.main()
